/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.services.components.script;

import com.vii.streamline.structures.collections.InnerMap;
import org.svenson.JSONProperty;

import java.util.Map;

/**
 * Specialized type to exchange scripts used by the
 * @see com.vii.brillien.services.components.script.Processor
 * presence type
 */
public class ScriptMessageProcessor {

    private String              name;
    private Double              version;
    private Map<String,String>  parameters;
    private String              script;


    {
        version     = 1.0;
        parameters  = new InnerMap<String, String>();
    }

    public ScriptMessageProcessor() {
    }

    public ScriptMessageProcessor(String name, String script) {
        this( name, new InnerMap<String, String>(), script );
    }
    public ScriptMessageProcessor(String name, Map<String, String> parameters, String script) {
        this( name, 1.0, parameters, script );
    }
    public ScriptMessageProcessor(String name, Double version, Map<String, String> parameters, String script) {
        this.name = name;
        this.version = version;
        this.parameters = parameters;
        this.script = script;
    }

    @JSONProperty( ignoreIfNull = true )
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @JSONProperty( ignoreIfNull = true )
    public Double getVersion() {
        return version;
    }

    public void setVersion(Double version) {
        this.version = version;
    }

    @JSONProperty( ignoreIfNull = true )
    public Map<String, String> getParameters() {
        return parameters;
    }

    public void setParameters(Map<String, String> parameters) {
        this.parameters = parameters;
    }

    @JSONProperty( ignoreIfNull = true )
    public String getScript() {
        return script;
    }

    public void setScript(String script) {
        this.script = script;
    }

    @Override
    public String toString() {
        return "ScriptMessageProcessor{" +
                "name='" + name + '\'' +
                ", version=" + version +
                ", script='" + script + '\'' +
                '}';
    }
}

