/*
 * Copyright (c) 2012 Imre Fazekas.
 *  All rights reserved.
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *  list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *  this list of conditions and the following disclaimer in the documentation
 *  and/or other materials provided with the distribution.
 *  Neither the name of the Brillien nor the names of its
 *  terms and concepts may be used to endorse or promote products derived from this
 *  software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.services.jsonrpc;

import com.vii.brillien.core.management.BrillienServices;
import com.vii.brillien.ignition.BrillienContext;
import com.vii.brillien.ignition.transport.BrillienMediator;
import com.vii.brillien.kernel.BrillienException;
import com.vii.streamline.apikey.ApiKeyValidator;
import com.vii.streamline.json.JsonRpcListener;
import com.vii.streamline.jsonrpc.JsonRpcDispatcherServlet;
import com.vii.streamline.sso.SessionValidator;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;

/**
 * Specialized JsonRpcDispatcherServlet to able to delegate JsonRpc calls to presences.
 */
public class BrillienJsonRpcServlet extends JsonRpcDispatcherServlet {

    /**
     * Singleton instance to be published by the webserver
     */
    protected static BrillienJsonRpcServlet             servletInstanceJson;

    /**
     * mediator used to communicate with presences
     */
    protected BrillienMediator                          mediator;

    static{
        servletInstanceJson = new BrillienJsonRpcServlet();
    }

    /**
     * Following the singleton model, retrieves the single instance created
     */
    public static BrillienJsonRpcServlet getServletInstance() {
        return servletInstanceJson;
    }

    @Override
    protected boolean checkApiKeyRequired(){
        boolean apiKeyRequired = super.checkApiKeyRequired();

        return  BrillienServices.liaison != null
                ? BrillienServices.liaison.getApiKeyPresenceName() != null
                : apiKeyRequired;
    }

    @Override
    protected boolean checkSsoRequired(){
        boolean ssoRequired = super.checkSsoRequired();

        return  BrillienServices.liaison != null
                ? BrillienServices.liaison.getSSOPresenceName() != null
                : ssoRequired;
    }

    @Override
    protected ApiKeyValidator getApiKeyValidator() throws ServletException {
        try {
            return (ApiKeyValidator)BrillienServices.getPresence( BrillienServices.liaison, BrillienServices.liaison.getApiKeyPresenceName() );
        } catch (BrillienException e) {
            throw new ServletException( e );
        }
    }

    @Override
    protected SessionValidator getSessionValidator() throws ServletException {
        try {
            return (SessionValidator)BrillienServices.getPresence( BrillienServices.liaison, BrillienServices.liaison.getSSOPresenceName() );
        } catch (BrillienException e) {
            throw new ServletException( e );
        }
    }


    /**
     * Init method to set up mediator connectivity when the servlet is initiated by the web server
     */
    protected void setupMediatorContext( ServletConfig servletConfig ){

    }


    @Override
    protected JsonRpcListener getJsonRpcListener() throws ServletException {
        if( listener == null ){
            String username = getServletConfig().getInitParameter("username");
            String password = getServletConfig().getInitParameter("password");

            if( username == null ){
                username = "dispatcher" + this.getClass().getSimpleName(); password = "dispatcher";
            }

            try {
                if( mediator == null ){
                    setupMediatorContext( getServletConfig() );
                    mediator = BrillienContext.newMediator( username, password );
                    mediator.connect();
                }
            } catch (BrillienException e) {
                throw new ServletException( e );
            }

            listener = new BrillienJsonRpcListener( ssoRequired, logger, mediator );
        }
        return listener;
    }

    @Override
    public void destroy() {
        try {
            if( mediator != null )
                mediator.disconnect();
        } catch (BrillienException e) { }
        BrillienContext.cleanup();
    }

}
